@@ -28,11 +28,18 @@ from datafusion.expr import (
     CaseBuilder,
     Expr,
     SortExpr,
+    SortKey,
     WindowFrame,
     expr_list_to_raw_expr_list,
     sort_list_to_raw_sort_list,
+    sort_or_default,
 )
 
+try:
+    from warnings import deprecated  # Python 3.13+
+except ImportError:
+    from typing_extensions import deprecated  # Python 3.12
+
 if TYPE_CHECKING:
     from datafusion.context import SessionContext
 
@@ -425,12 +432,15 @@ def when(when: Expr, then: Expr) -> CaseBuilder:
     return CaseBuilder(f.when(when.expr, then.expr))
 
 
+@deprecated("Prefer to call Expr.over() instead")
 def window(
     name: str,
     args: list[Expr],
-    partition_by: list[Expr] | None = None,
-    order_by: list[Expr | SortExpr] | None = None,
+    partition_by: list[Expr] | Expr | None = None,
+    order_by: list[SortKey] | SortKey | None = None,
     window_frame: WindowFrame | None = None,
+    filter: Expr | None = None,
+    distinct: bool = False,
     ctx: SessionContext | None = None,
 ) -> Expr:
     """Creates a new Window function expression.
@@ -440,13 +450,29 @@ def window(
     lag use::
 
         df.select(functions.lag(col("a")).partition_by(col("b")).build())
+
+    The ``order_by`` parameter accepts column names or expressions, e.g.::
+
+        window("lag", [col("a")], order_by="ts")
     """
     args = [a.expr for a in args]
-    partition_by = expr_list_to_raw_expr_list(partition_by)
+    partition_by_raw = expr_list_to_raw_expr_list(partition_by)
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     window_frame = window_frame.window_frame if window_frame is not None else None
     ctx = ctx.ctx if ctx is not None else None
-    return Expr(f.window(name, args, partition_by, order_by_raw, window_frame, ctx))
+    filter_raw = filter.expr if filter is not None else None
+    return Expr(
+        f.window(
+            name,
+            args,
+            partition_by=partition_by_raw,
+            order_by=order_by_raw,
+            window_frame=window_frame,
+            ctx=ctx,
+            filter=filter_raw,
+            distinct=distinct,
+        )
+    )
 
 
 # scalar functions
@@ -1659,7 +1685,7 @@ def approx_median(expression: Expr, filter: Optional[Expr] = None) -> Expr:
 
 
 def approx_percentile_cont(
-    expression: Expr,
+    sort_expression: Expr | SortExpr,
     percentile: float,
     num_centroids: Optional[int] = None,
     filter: Optional[Expr] = None,
@@ -1673,28 +1699,33 @@ def approx_percentile_cont(
     between two of the values.
 
     This function uses the [t-digest](https://arxiv.org/abs/1902.04023) algorithm to
-    compute the percentil. You can limit the number of bins used in this algorithm by
+    compute the percentile. You can limit the number of bins used in this algorithm by
     setting the ``num_centroids`` parameter.
 
     If using the builder functions described in ref:`_aggregation` this function ignores
     the options ``order_by``, ``null_treatment``, and ``distinct``.
 
     Args:
-        expression: Values for which to find the approximate percentile
+        sort_expression: Values for which to find the approximate percentile
         percentile: This must be between 0.0 and 1.0, inclusive
         num_centroids: Max bin size for the t-digest algorithm
         filter: If provided, only compute against rows for which the filter is True
     """
+    sort_expr_raw = sort_or_default(sort_expression)
     filter_raw = filter.expr if filter is not None else None
     return Expr(
         f.approx_percentile_cont(
-            expression.expr, percentile, num_centroids=num_centroids, filter=filter_raw
+            sort_expr_raw, percentile, num_centroids=num_centroids, filter=filter_raw
         )
     )
 
 
 def approx_percentile_cont_with_weight(
-    expression: Expr, weight: Expr, percentile: float, filter: Optional[Expr] = None
+    sort_expression: Expr | SortExpr,
+    weight: Expr,
+    percentile: float,
+    num_centroids: Optional[int] = None,
+    filter: Optional[Expr] = None,
 ) -> Expr:
     """Returns the value of the weighted approximate percentile.
 
@@ -1705,16 +1736,22 @@ def approx_percentile_cont_with_weight(
     the options ``order_by``, ``null_treatment``, and ``distinct``.
 
     Args:
-        expression: Values for which to find the approximate percentile
+        sort_expression: Values for which to find the approximate percentile
         weight: Relative weight for each of the values in ``expression``
         percentile: This must be between 0.0 and 1.0, inclusive
+        num_centroids: Max bin size for the t-digest algorithm
         filter: If provided, only compute against rows for which the filter is True
 
     """
+    sort_expr_raw = sort_or_default(sort_expression)
     filter_raw = filter.expr if filter is not None else None
     return Expr(
         f.approx_percentile_cont_with_weight(
-            expression.expr, weight.expr, percentile, filter=filter_raw
+            sort_expr_raw,
+            weight.expr,
+            percentile,
+            num_centroids=num_centroids,
+            filter=filter_raw,
         )
     )
 
@@ -1723,7 +1760,7 @@ def array_agg(
     expression: Expr,
     distinct: bool = False,
     filter: Optional[Expr] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Aggregate values into an array.
 
@@ -1738,7 +1775,11 @@ def array_agg(
         expression: Values to combine into an array
         distinct: If True, a single entry for each distinct value will be in the result
         filter: If provided, only compute against rows for which the filter is True
-        order_by: Order the resultant array values
+        order_by: Order the resultant array values. Accepts column names or expressions.
+
+    For example::
+
+        df.select(array_agg(col("a"), order_by="b"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
@@ -2222,7 +2263,7 @@ def regr_syy(
 def first_value(
     expression: Expr,
     filter: Optional[Expr] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
     null_treatment: NullTreatment = NullTreatment.RESPECT_NULLS,
 ) -> Expr:
     """Returns the first value in a group of values.
@@ -2235,8 +2276,13 @@ def first_value(
     Args:
         expression: Argument to perform bitwise calculation on
         filter: If provided, only compute against rows for which the filter is True
-        order_by: Set the ordering of the expression to evaluate
+        order_by: Set the ordering of the expression to evaluate. Accepts
+            column names or expressions.
         null_treatment: Assign whether to respect or ignore null values.
+
+    For example::
+
+        df.select(first_value(col("a"), order_by="ts"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
@@ -2254,7 +2300,7 @@ def first_value(
 def last_value(
     expression: Expr,
     filter: Optional[Expr] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
     null_treatment: NullTreatment = NullTreatment.RESPECT_NULLS,
 ) -> Expr:
     """Returns the last value in a group of values.
@@ -2267,8 +2313,13 @@ def last_value(
     Args:
         expression: Argument to perform bitwise calculation on
         filter: If provided, only compute against rows for which the filter is True
-        order_by: Set the ordering of the expression to evaluate
+        order_by: Set the ordering of the expression to evaluate. Accepts
+            column names or expressions.
         null_treatment: Assign whether to respect or ignore null values.
+
+    For example::
+
+        df.select(last_value(col("a"), order_by="ts"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
@@ -2287,7 +2338,7 @@ def nth_value(
     expression: Expr,
     n: int,
     filter: Optional[Expr] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
     null_treatment: NullTreatment = NullTreatment.RESPECT_NULLS,
 ) -> Expr:
     """Returns the n-th value in a group of values.
@@ -2301,8 +2352,13 @@ def nth_value(
         expression: Argument to perform bitwise calculation on
         n: Index of value to return. Starts at 1.
         filter: If provided, only compute against rows for which the filter is True
-        order_by: Set the ordering of the expression to evaluate
+        order_by: Set the ordering of the expression to evaluate. Accepts
+            column names or expressions.
         null_treatment: Assign whether to respect or ignore null values.
+
+    For example::
+
+        df.select(nth_value(col("a"), 2, order_by="ts"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
@@ -2407,15 +2463,15 @@ def lead(
     arg: Expr,
     shift_offset: int = 1,
     default_value: Optional[Any] = None,
-    partition_by: Optional[list[Expr]] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    partition_by: Optional[list[Expr] | Expr] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Create a lead window function.
 
     Lead operation will return the argument that is in the next shift_offset-th row in
     the partition. For example ``lead(col("b"), shift_offset=3, default_value=5)`` will
     return the 3rd following value in column ``b``. At the end of the partition, where
-    no futher values can be returned it will return the default value of 5.
+    no further values can be returned it will return the default value of 5.
 
     Here is an example of both the ``lead`` and :py:func:`datafusion.functions.lag`
     functions on a simple DataFrame::
@@ -2437,14 +2493,17 @@ def lead(
         shift_offset: Number of rows following the current row.
         default_value: Value to return if shift_offet row does not exist.
         partition_by: Expressions to partition the window frame on.
-        order_by: Set ordering within the window frame.
+        order_by: Set ordering within the window frame. Accepts
+            column names or expressions.
+
+    For example::
+
+        lead(col("b"), order_by="ts")
     """
     if not isinstance(default_value, pa.Scalar) and default_value is not None:
         default_value = pa.scalar(default_value)
 
-    partition_cols = (
-        [col.expr for col in partition_by] if partition_by is not None else None
-    )
+    partition_by_raw = expr_list_to_raw_expr_list(partition_by)
     order_by_raw = sort_list_to_raw_sort_list(order_by)
 
     return Expr(
@@ -2452,7 +2511,7 @@ def lead(
             arg.expr,
             shift_offset,
             default_value,
-            partition_by=partition_cols,
+            partition_by=partition_by_raw,
             order_by=order_by_raw,
         )
     )
@@ -2462,14 +2521,14 @@ def lag(
     arg: Expr,
     shift_offset: int = 1,
     default_value: Optional[Any] = None,
-    partition_by: Optional[list[Expr]] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    partition_by: Optional[list[Expr] | Expr] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Create a lag window function.
 
     Lag operation will return the argument that is in the previous shift_offset-th row
     in the partition. For example ``lag(col("b"), shift_offset=3, default_value=5)``
-    will return the 3rd previous value in column ``b``. At the beginnig of the
+    will return the 3rd previous value in column ``b``. At the beginning of the
     partition, where no values can be returned it will return the default value of 5.
 
     Here is an example of both the ``lag`` and :py:func:`datafusion.functions.lead`
@@ -2489,14 +2548,17 @@ def lag(
         shift_offset: Number of rows before the current row.
         default_value: Value to return if shift_offet row does not exist.
         partition_by: Expressions to partition the window frame on.
-        order_by: Set ordering within the window frame.
+        order_by: Set ordering within the window frame. Accepts
+            column names or expressions.
+
+    For example::
+
+        lag(col("b"), order_by="ts")
     """
     if not isinstance(default_value, pa.Scalar):
         default_value = pa.scalar(default_value)
 
-    partition_cols = (
-        [col.expr for col in partition_by] if partition_by is not None else None
-    )
+    partition_by_raw = expr_list_to_raw_expr_list(partition_by)
     order_by_raw = sort_list_to_raw_sort_list(order_by)
 
     return Expr(
@@ -2504,15 +2566,15 @@ def lag(
             arg.expr,
             shift_offset,
             default_value,
-            partition_by=partition_cols,
+            partition_by=partition_by_raw,
             order_by=order_by_raw,
         )
     )
 
 
 def row_number(
-    partition_by: Optional[list[Expr]] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    partition_by: Optional[list[Expr] | Expr] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Create a row number window function.
 
@@ -2531,30 +2593,33 @@ def row_number(
 
     Args:
         partition_by: Expressions to partition the window frame on.
-        order_by: Set ordering within the window frame.
+        order_by: Set ordering within the window frame. Accepts
+            column names or expressions.
+
+    For example::
+
+        row_number(order_by="points")
     """
-    partition_cols = (
-        [col.expr for col in partition_by] if partition_by is not None else None
-    )
+    partition_by_raw = expr_list_to_raw_expr_list(partition_by)
     order_by_raw = sort_list_to_raw_sort_list(order_by)
 
     return Expr(
         f.row_number(
-            partition_by=partition_cols,
+            partition_by=partition_by_raw,
             order_by=order_by_raw,
         )
     )
 
 
 def rank(
-    partition_by: Optional[list[Expr]] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    partition_by: Optional[list[Expr] | Expr] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Create a rank window function.
 
     Returns the rank based upon the window order. Consecutive equal values will receive
     the same rank, but the next different value will not be consecutive but rather the
-    number of rows that preceed it plus one. This is similar to Olympic medals. If two
+    number of rows that precede it plus one. This is similar to Olympic medals. If two
     people tie for gold, the next place is bronze. There would be no silver medal. Here
     is an example of a dataframe with a window ordered by descending ``points`` and the
     associated rank.
@@ -2572,24 +2637,27 @@ def rank(
 
     Args:
         partition_by: Expressions to partition the window frame on.
-        order_by: Set ordering within the window frame.
+        order_by: Set ordering within the window frame. Accepts
+            column names or expressions.
+
+    For example::
+
+        rank(order_by="points")
     """
-    partition_cols = (
-        [col.expr for col in partition_by] if partition_by is not None else None
-    )
+    partition_by_raw = expr_list_to_raw_expr_list(partition_by)
     order_by_raw = sort_list_to_raw_sort_list(order_by)
 
     return Expr(
         f.rank(
-            partition_by=partition_cols,
+            partition_by=partition_by_raw,
             order_by=order_by_raw,
         )
     )
 
 
 def dense_rank(
-    partition_by: Optional[list[Expr]] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    partition_by: Optional[list[Expr] | Expr] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Create a dense_rank window function.
 
@@ -2608,24 +2676,27 @@ def dense_rank(
 
     Args:
         partition_by: Expressions to partition the window frame on.
-        order_by: Set ordering within the window frame.
+        order_by: Set ordering within the window frame. Accepts
+            column names or expressions.
+
+    For example::
+
+        dense_rank(order_by="points")
     """
-    partition_cols = (
-        [col.expr for col in partition_by] if partition_by is not None else None
-    )
+    partition_by_raw = expr_list_to_raw_expr_list(partition_by)
     order_by_raw = sort_list_to_raw_sort_list(order_by)
 
     return Expr(
         f.dense_rank(
-            partition_by=partition_cols,
+            partition_by=partition_by_raw,
             order_by=order_by_raw,
         )
     )
 
 
 def percent_rank(
-    partition_by: Optional[list[Expr]] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    partition_by: Optional[list[Expr] | Expr] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Create a percent_rank window function.
 
@@ -2645,29 +2716,32 @@ def percent_rank(
 
     Args:
         partition_by: Expressions to partition the window frame on.
-        order_by: Set ordering within the window frame.
+        order_by: Set ordering within the window frame. Accepts
+            column names or expressions.
+
+    For example::
+
+        percent_rank(order_by="points")
     """
-    partition_cols = (
-        [col.expr for col in partition_by] if partition_by is not None else None
-    )
+    partition_by_raw = expr_list_to_raw_expr_list(partition_by)
     order_by_raw = sort_list_to_raw_sort_list(order_by)
 
     return Expr(
         f.percent_rank(
-            partition_by=partition_cols,
+            partition_by=partition_by_raw,
             order_by=order_by_raw,
         )
     )
 
 
 def cume_dist(
-    partition_by: Optional[list[Expr]] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    partition_by: Optional[list[Expr] | Expr] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Create a cumulative distribution window function.
 
     This window function is similar to :py:func:`rank` except that the returned values
-    are the ratio of the row number to the total numebr of rows. Here is an example of a
+    are the ratio of the row number to the total number of rows. Here is an example of a
     dataframe with a window ordered by descending ``points`` and the associated
     cumulative distribution::
 
@@ -2682,16 +2756,19 @@ def cume_dist(
 
     Args:
         partition_by: Expressions to partition the window frame on.
-        order_by: Set ordering within the window frame.
+        order_by: Set ordering within the window frame. Accepts
+            column names or expressions.
+
+    For example::
+
+        cume_dist(order_by="points")
     """
-    partition_cols = (
-        [col.expr for col in partition_by] if partition_by is not None else None
-    )
+    partition_by_raw = expr_list_to_raw_expr_list(partition_by)
     order_by_raw = sort_list_to_raw_sort_list(order_by)
 
     return Expr(
         f.cume_dist(
-            partition_by=partition_cols,
+            partition_by=partition_by_raw,
             order_by=order_by_raw,
         )
     )
@@ -2699,8 +2776,8 @@ def cume_dist(
 
 def ntile(
     groups: int,
-    partition_by: Optional[list[Expr]] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    partition_by: Optional[list[Expr] | Expr] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Create a n-tile window function.
 
@@ -2723,17 +2800,20 @@ def ntile(
     Args:
         groups: Number of groups for the n-tile to be divided into.
         partition_by: Expressions to partition the window frame on.
-        order_by: Set ordering within the window frame.
+        order_by: Set ordering within the window frame. Accepts
+            column names or expressions.
+
+    For example::
+
+        ntile(3, order_by="points")
     """
-    partition_cols = (
-        [col.expr for col in partition_by] if partition_by is not None else None
-    )
+    partition_by_raw = expr_list_to_raw_expr_list(partition_by)
     order_by_raw = sort_list_to_raw_sort_list(order_by)
 
     return Expr(
         f.ntile(
             Expr.literal(groups).expr,
-            partition_by=partition_cols,
+            partition_by=partition_by_raw,
             order_by=order_by_raw,
         )
     )
@@ -2743,12 +2823,12 @@ def string_agg(
     expression: Expr,
     delimiter: str,
     filter: Optional[Expr] = None,
-    order_by: Optional[list[Expr | SortExpr]] = None,
+    order_by: Optional[list[SortKey] | SortKey] = None,
 ) -> Expr:
     """Concatenates the input strings.
 
     This aggregate function will concatenate input strings, ignoring null values, and
-    seperating them with the specified delimiter. Non-string values will be converted to
+    separating them with the specified delimiter. Non-string values will be converted to
     their string equivalents.
 
     If using the builder functions described in ref:`_aggregation` this function ignores
@@ -2758,7 +2838,12 @@ def string_agg(
         expression: Argument to perform bitwise calculation on
         delimiter: Text to place between each value of expression
         filter: If provided, only compute against rows for which the filter is True
-        order_by: Set the ordering of the expression to evaluate
+        order_by: Set the ordering of the expression to evaluate. Accepts
+            column names or expressions.
+
+    For example::
+
+        df.select(string_agg(col("a"), ",", order_by="b"))
     """
     order_by_raw = sort_list_to_raw_sort_list(order_by)
     filter_raw = filter.expr if filter is not None else None
