@@ -27,6 +27,7 @@ from datafusion import (
     SessionConfig,
     SessionContext,
     SQLOptions,
+    Table,
     column,
     literal,
 )
@@ -311,7 +312,7 @@ def test_register_table(ctx, database):
     assert public.names() == {"csv", "csv1", "csv2", "csv3"}
 
 
-def test_read_table(ctx, database):
+def test_read_table_from_catalog(ctx, database):
     default = ctx.catalog()
     public = default.schema("public")
     assert public.names() == {"csv", "csv1", "csv2"}
@@ -321,6 +322,25 @@ def test_read_table(ctx, database):
     table_df.show()
 
 
+def test_read_table_from_df(ctx):
+    df = ctx.from_pydict({"a": [1, 2]})
+    result = ctx.read_table(df).collect()
+    assert [b.to_pydict() for b in result] == [{"a": [1, 2]}]
+
+
+def test_read_table_from_dataset(ctx):
+    batch = pa.RecordBatch.from_arrays(
+        [pa.array([1, 2, 3]), pa.array([4, 5, 6])],
+        names=["a", "b"],
+    )
+    dataset = ds.dataset([batch])
+
+    result = ctx.read_table(dataset).collect()
+
+    assert result[0].column(0) == pa.array([1, 2, 3])
+    assert result[0].column(1) == pa.array([4, 5, 6])
+
+
 def test_deregister_table(ctx, database):
     default = ctx.catalog()
     public = default.schema("public")
@@ -330,6 +350,40 @@ def test_deregister_table(ctx, database):
     assert public.names() == {"csv1", "csv2"}
 
 
+def test_register_table_from_dataframe(ctx):
+    df = ctx.from_pydict({"a": [1, 2]})
+    ctx.register_table("df_tbl", df)
+    result = ctx.sql("SELECT * FROM df_tbl").collect()
+    assert [b.to_pydict() for b in result] == [{"a": [1, 2]}]
+
+
+def test_register_table_from_dataframe_into_view(ctx):
+    df = ctx.from_pydict({"a": [1, 2]})
+    table = df.into_view()
+    assert isinstance(table, Table)
+    ctx.register_table("view_tbl", table)
+    result = ctx.sql("SELECT * FROM view_tbl").collect()
+    assert [b.to_pydict() for b in result] == [{"a": [1, 2]}]
+
+
+def test_table_from_dataframe(ctx):
+    df = ctx.from_pydict({"a": [1, 2]})
+    table = Table(df)
+    assert isinstance(table, Table)
+    ctx.register_table("from_dataframe_tbl", table)
+    result = ctx.sql("SELECT * FROM from_dataframe_tbl").collect()
+    assert [b.to_pydict() for b in result] == [{"a": [1, 2]}]
+
+
+def test_table_from_dataframe_internal(ctx):
+    df = ctx.from_pydict({"a": [1, 2]})
+    table = Table(df.df)
+    assert isinstance(table, Table)
+    ctx.register_table("from_internal_dataframe_tbl", table)
+    result = ctx.sql("SELECT * FROM from_internal_dataframe_tbl").collect()
+    assert [b.to_pydict() for b in result] == [{"a": [1, 2]}]
+
+
 def test_register_dataset(ctx):
     # create a RecordBatch and register it as a pyarrow.dataset.Dataset
     batch = pa.RecordBatch.from_arrays(
