@@ -23,7 +23,7 @@ use super::logical_node::LogicalNode;
 use crate::common::df_schema::PyDFSchema;
 use crate::sql::logical::PyLogicalPlan;
 
-#[pyclass(name = "Analyze", module = "datafusion.expr", subclass)]
+#[pyclass(frozen, name = "Analyze", module = "datafusion.expr", subclass)]
 #[derive(Clone)]
 pub struct PyAnalyze {
     analyze: Analyze,
@@ -69,7 +69,7 @@ impl PyAnalyze {
     }
 
     fn __repr__(&self) -> PyResult<String> {
-        Ok(format!("Analyze({})", self))
+        Ok(format!("Analyze({self})"))
     }
 }
 
