@@ -28,7 +28,7 @@ use crate::common::df_schema::PyDFSchema;
 use super::logical_node::LogicalNode;
 use crate::sql::logical::PyLogicalPlan;
 
-#[pyclass(name = "DropView", module = "datafusion.expr", subclass)]
+#[pyclass(frozen, name = "DropView", module = "datafusion.expr", subclass)]
 #[derive(Clone)]
 pub struct PyDropView {
     drop: DropView,
