@@ -18,7 +18,7 @@
 use datafusion::logical_expr::GroupingSet;
 use pyo3::prelude::*;
 
-#[pyclass(name = "GroupingSet", module = "datafusion.expr", subclass)]
+#[pyclass(frozen, name = "GroupingSet", module = "datafusion.expr", subclass)]
 #[derive(Clone)]
 pub struct PyGroupingSet {
     grouping_set: GroupingSet,
